#!/usr/bin/env python
# encoding: utf-8

from django.template import Node, Library
from djengel.tags.models import Tag
from django import template

register = Library()

class TagListNode(Node):
    def __init__(self, variable):
        self.variable = variable

    def render(self, context):
        context[self.variable] = Tag.objects.all().order_by('norm_value','value')
        return ''

def do_get_tag_list(parser, token):
    """
    {% get_tag_list as tag_list %}

    タグの一覧をテンプレートで利用可能にします。
    """
    bits = token.contents.split()
    if len(bits) != 3:
        raise template.TemplateSyntaxError, "'%s' tag takes two arguments" % bits[0]
    if bits[1] != "as":
        raise template.TemplateSyntaxError, "First argument to '%s' tag must be 'as'" % bits[0]
    return TagListNode(bits[2])
register.tag('get_tag_list', do_get_tag_list)


class AmmountTotalNode(Node) :
    def __init__(self, variable):
        self.variable = variable

    def render(self, context):
        object_list = context['object_list']
        context[self.variable] = sum([o.amount for o in object_list])
        return ''

def do_get_amount_total(parser, token) :
    """
    {% get_amount_total as total %}
    """
    bits = token.contents.split()
    if len(bits) != 3:
        raise template.TemplateSyntaxError, "'%s' tag takes 2 arguments" % bits[0]
    return AmmountTotalNode(bits[2])

register.tag('get_amount_total', do_get_amount_total)
