# -*- coding: utf8 -*-
from datetime import datetime, timedelta
import calendar

from django.utils.translation import gettext_lazy as _
from django.contrib.auth.decorators import login_required, user_passes_test
from django.shortcuts import render_to_response
from django.http import HttpResponse, HttpResponseRedirect
from django.views.generic.list_detail import object_list as generic_object_list

from djengel.squander.models import ExpenseItem, ExpenseItemSummary, Outgoing

#PyChart が必要です。(Pychart には GhostScript が必要です。)
#http://home.gna.org/pychart/
from pychart import theme, canvas, area, legend, pie_plot, fill_style, chart_data, axis, line_plot
import tocslib #PyCharのサンプルについてくる定数スクリプトを、プロジェクト直下においてください。

@user_passes_test(lambda u: u.has_perm('squander.can_view'), login_url='/accounts/login/')
def index(request) :
    """
       / へのアクセスを当月を指定して単月の状況(/summary_month/yyyy/mm/)にリダイレクトします。
    """
    dn = datetime.now()
    return HttpResponseRedirect('/djengel/summary/%04d/%02d/' % (dn.year, dn.month))

@user_passes_test(lambda u: u.has_perm('squander.can_view'), login_url='/accounts/login/')
def summary_month(request, year, month) :
    year, month = int(year) ,int(month)
    prev_month, next_month = get_prev_next(year, month)

    object_list = ExpenseItem.summary_objects.get_summary_by_month(year, month)
    total = max_amount_total = 0
    for o in object_list :
        total += o.amount
        max_amount_total += o.max_amount
    year_month = '%04d/%02d/' % (year, month)
    return render_to_response('squander/expenseitem_list.html',locals())


@user_passes_test(lambda u: u.has_perm('squander.can_view'), login_url='/accounts/login/')
def ratio_graph_specified_month(request, year, month) :
    """
       指定された月の費目比率円グラフを返します。
    """
    data = ExpenseItem.summary_objects.get_summary_by_month(year, month)
    return _create_month_summary_data(data)

def _create_month_summary_data(data) :
    """
       入力データから円グラフを作成します。
    """
    input_data = []
    if len(data) == 0 :
	    input_data.append(('none', '100'))
    input_data = [('%d' % summ.expense_id, summ.amount) for summ in data]

    ofst = [15 for t in input_data]

    theme.get_options()
    theme.use_color = True
    theme.output_format="png"
    theme.reinitialize()
    
    ar = area.T(size=(300,300), legend=None,
             x_grid_style=None, y_grid_style=None)
    ar.add_plot(pie_plot.T(data = input_data,
                           arc_offsets=ofst))

    response = HttpResponse(mimetype='image/png')
    can = canvas.init(response)
    ar.draw(can)
    can.close()
    return response

# 3MONTH SUMMARY AND LINE GRAPH

@user_passes_test(lambda u: u.has_perm('squander.can_view'), login_url='/accounts/login/')
def recent(request) :
    """
       当月からマイナス recent_before ヶ月前までを指定して、直近の推移状況へとリダイレクトします。
    """
    recent_before = 2 #TODO move magic no to settings
    dn = datetime.now()
    year_s = dn.year
    month_s = dn.month - recent_before
    year_e  = dn.year
    month_e = dn.month
    if dn.month - recent_before < 1 :
        year_s = dn.year - 1
        month_s = dn.month + 12 - recent_before
    return HttpResponseRedirect('/djengel/recent/%04d/%02d/%04d/%02d/' % (year_s, month_s, year_e, month_e))
    
@user_passes_test(lambda u: u.has_perm('squander.can_view'), login_url='/accounts/login/')
def recent_expense(request, year_s, month_s, year_e, month_e) :
    """
      指定期間の推移状況を表示します。
    """
    data = ExpenseItem.summary_objects.get_summary_by_year_month('%s/%s' % (year_s, month_s), '%s/%s' % (year_e, month_e))
    expense_list = ExpenseItem.objects.order_by('id')
    ym_list = get_year_month_list(int(year_s), int(month_s), int(year_e), int(month_e))
    data_dict = {}
    for d in data :
        data_dict['%d-%s' % (d.expense_id, d.pay_date)] = d

    year_month_list = [ym for ym in ym_list]

    sum_none_exist = ExpenseItemSummary()
    data_list = []
    ym_total = [0] * len(ym_list)
    for exp in expense_list :
        exp_line = [exp.id, exp.name]
        i = 0
        for ym in ym_list :
            amount = data_dict.get('%d-%s' % (exp.id, ym), sum_none_exist).amount
            exp_line.append('%s' % amount)
            ym_total[i] = ym_total[i] + amount
            i+=1
        data_list.append(exp_line)
    ym_range = '%s/%s/%s/%s/' % (year_s, month_s, year_e, month_e)
    return render_to_response('squander/recent_list.html', locals())

@user_passes_test(lambda u: u.has_perm('squander.can_view'), login_url='/accounts/login/')
def line_graph(request, year_s, month_s, year_e, month_e) :
    """
      指定期間の折れ線グラフを表示します。
    """
    stl = ["PN","PRV","PR","SP","D4","D2","D1","R","S4","S2","S1"]
    data = ExpenseItem.summary_objects.get_summary_by_year_month('%s/%s' % (year_s, month_s), '%s/%s' % (year_e, month_e))
    expense_list = ExpenseItem.objects.order_by('id')
    ym_list = get_year_month_list(int(year_s), int(month_s), int(year_e), int(month_e))

    data_dict = {}
    for d in data :
        data_dict['%d-%s' % (d.expense_id, d.pay_date)] = d

    real_data = []
    sum_none_exist = ExpenseItemSummary()

    max_amount = 1
    count = 0
    for ym in ym_list :
        count += 1
        line_data = []
        line_data.append('%s' % count)
        for exp in expense_list :
            amount = data_dict.get('%d-%s' % (exp.id, ym), sum_none_exist).amount
            if amount > max_amount :
                max_amount = amount
            line_data.append('%s' % amount)
        real_data.append(' '.join(line_data))

    theme.get_options()
    theme.use_color = True
    theme.output_format="png"
    theme.reinitialize()
    

    data = chart_data.read_str(' ', *real_data)


    plot_list = []

    ar = area.T(size=(tocslib.width2, tocslib.height),
            x_axis = None,
            y_axis = axis.Y(label="/bAmount", tic_interval=max_amount/10,
                            label_offset=(tocslib.xlabel_offset, None)),
            legend=legend.T())

    yc = 0
    for e in expense_list :
        yc += 1
        ar.add_plot(line_plot.T(label='%d' % e.id,
                        tick_mark=tocslib.get_tick_mark(stl[yc%len(stl)]),
                        line_style=tocslib.get_line_style(stl[yc%len(stl) - 1]),
                        data=data, ycol=yc))

    response = HttpResponse(mimetype='image/png')
    can = canvas.init(response)
    ar.draw(can)
    can.close()
    return response

def breakdown(request) :
    """
      当月を指定して、明細一覧へリダイレクトします。
    """
    dn = datetime.now()
    return HttpResponseRedirect('/djengel/breakdown/%04d/%02d/' % (dn.year, dn.month))

def breakdown_by_year_month(request, year='1970', month='1') :
    """
      指定月の明細一覧を表示します。
    """
    year, month = int(year), int(month)
    prev_month, next_month = get_prev_next(year, month)

    tag_ids = []
    for v in request.GET.lists() : #配列を受け取る場合には、GETの名前指定ではいけない(最後に指定されている値しかとれないため)
        if v[0] == 'tag_ids' :
            tag_ids = v[1]
            print tag_ids

    year_month = '%04d/%02d' % (year, month)
    queryset = Outgoing.objects.filter(pay_date__year=year,
                                       pay_date__month=month).order_by('pay_date').select_related()

    #おおっと、ManyToManyのAnd検索はどうやるんだ？このやりかたがヤバいのは間違いない。
    result = []
    for o in queryset :
        o_tag_ids = [t.id for t in o.tags.all()]
        if len([id for id in tag_ids if int(id) in o_tag_ids]) == len(tag_ids) :
            result.append(o)

    return render_to_response('squander/outgoing_list.html', {'object_list': result, 'year_month': year_month, 'tag_ids': tag_ids,
                                                       'prev_month': prev_month, 'next_month': next_month,
                                                       'get_query': request.GET.urlencode()})

def get_year_month_list(year_s, month_s, year_e, month_e) :
    """
      二つの指定された年月から年月(yyyy/mm)の配列を返します。
      eg. [200611, 200612, 200701] = get_year_month_list(2006,11, 2007, 1)
    """
    list = []
    result = {}
    s = int('%04d%02d' % (year_s, month_s))
    e = int('%04d%02d' % (year_e, month_e))
    if s >= e :
        return {'%04d/%02d' % (year_s, month_s): []}
    if year_s == year_e :
        list += ['%04d/%02d' % (year_s, m) for m in range(month_s, month_e + 1)]
    else :
        list += ['%04d/%02d' % (year_s, m) for m in range(month_s, 13)]
        for y in range(year_s + 1, year_e) :
            list += ['%04d/%02d' % (y, m) for m in range(1, 13)]
        list += ['%04d/%02d' % (year_e, m) for m in range(1, month_e + 1)]
    return list

def get_prev_next(year=1970, month=1) :
    """
      指定の年月から前月と次月を返します
      eg. ('2006/11/', '2007/01/') = get_prev_next(2006,12)
    """
    dn = datetime(year, month, 15)
    dlt = timedelta(days=20)
    prev_d = dn - dlt
    next_d = dn + dlt
    prev_month = '%04d/%02d/' % (prev_d.year, prev_d.month)
    next_month = '%04d/%02d/' % (next_d.year, next_d.month)
    return (prev_month, next_month)
